// USB2 API DLL example program
//
// USB2_DLL_Demo.cpp
//
// by:	I.Robinson, T.Rowling, PassMark Software
// date: 1-Feb-2005
//
// Purpose:
//	Demo software to show how to use the USB2 API. 
//
// Copyright PassMark 2009
// www.passmark.com
#pragma warning(disable : 4996)	//Disable CRT insecure warnings for strcpy and sprintf in visual studio 2005

//Includes
#include <windows.h>
#include <stdio.h>
#include <process.h>
#include <commctrl.h>
#include "resource.h"
#include <winioctl.h>
#include <time.h>

#include "USB2_DLL_Demo.h"
#include "USB2DLL.h"

HINSTANCE hGInstance = NULL;

//About.cpp
INT_PTR CALLBACK AboutDlgProc (HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam);

//Function prototypes
BOOL CALLBACK WndProc (HWND, UINT, WPARAM, LPARAM) ;
void InitDialog (HWND hwnd);

//
//Globals for this module
//
HINSTANCE	g_hInstance;	
HWND		g_hwndMain;	


int		Mode;
bool	connectedPlugs[MAX_USB_DEV_NUMBER];

#define NUMUSBCOLUMNS 5 

int usb_columns_width[NUMUSBCOLUMNS] = {
	100,
	150,
	100,
	100,
	150
};

char* usb_columns[NUMUSBCOLUMNS] = {
	"Serial", 
	"Description", 
	"Operations",
	"Errors",
	"Recovered Bus Errors"
};


//********************************
//Main windows procedure
//
//Code entry point
//********************************
int WINAPI WinMain (HINSTANCE hInst, HINSTANCE hPrevInstance,
                    PSTR szCmdLine, int iCmdShow)
{


	static char szAppName[] = "USB2DLLDEMO" ;	// Refernece to recource name
	MSG         msg ;
	OSVERSIONINFO	OSversion;
	
	//Initialise some globals
	g_hInstance = hInst;

	//Get OS version
	OSversion.dwOSVersionInfoSize = sizeof (OSVERSIONINFO);
	GetVersionEx (&OSversion);
	
	INITCOMMONCONTROLSEX commCtrl;
	commCtrl.dwSize = sizeof(INITCOMMONCONTROLSEX);
	commCtrl.dwICC = ICC_WIN95_CLASSES ;
	InitCommonControlsEx(&commCtrl);
	
	//Create and display the window	
	g_hwndMain = CreateDialog (g_hInstance, "USB2DLLDEMO", 0, (DLGPROC)WndProc);

	ShowWindow (g_hwndMain, iCmdShow) ;
	InitDialog (g_hwndMain);

	while (GetMessage (&msg, NULL, 0, 0))
	{
		if (!IsDialogMessage(g_hwndMain,&msg)) {
			TranslateMessage (&msg) ;
			DispatchMessage (&msg) ;
		}
	}
	return msg.wParam ;
}


//********************************
//Windows callback procedure
//
//Handles all the messages for the dialog box
//********************************
BOOL CALLBACK WndProc (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam) 
{
	
	HANDLE  hDevice                         = NULL;

	BOOLEAN bResult                         = FALSE;
	int     nBytes                          = 0;
	PVOID   pvBuffer                        = 0;
	int     nItems                          = 0;
	HFONT   hFont                           = NULL;
	ULONG   ulLength                         = 0;
	HWND    hControl = NULL;
	
	
	// Get a handle to the output box
	switch(message)	
	{
	case WM_CREATE :
			break;
		
		
	case WM_TIMER :
		{
			//Update the listview 
			char tmpBuf[256]= {""};

			if(USB2TestIsRunning())
			{
				if(Mode == LOOPBACK)
				{
					int operations[MAX_USB_DEV_NUMBER];
					int errors[MAX_USB_DEV_NUMBER];
					HWND hwnd_LV_USB = GetDlgItem(g_hwndMain, IDC_PLUGLIST);		

					//call USB2GetLoopbackResults from USB2 DLL
					USB2GetLoopbackResults(operations, errors);
					
					//For each connected plug get the results
					int connected =0;
					for(int a=0; a<MAX_USB_DEV_NUMBER; a++)
					{
						if(connectedPlugs[a] == true)
						{
							sprintf(tmpBuf, "%d", operations[a]);
							ListView_SetItemText(hwnd_LV_USB, connected, 2, tmpBuf);
							sprintf(tmpBuf, "%d", errors[a]);
							ListView_SetItemText(hwnd_LV_USB, connected, 3, tmpBuf);
							//call USB2GetBusErrors from USB2 DLL
							sprintf(tmpBuf, "%d", USB2GetBusErrors(connected));
							ListView_SetItemText(hwnd_LV_USB, connected, 4, tmpBuf);
							connected++;
						}
					}
				}
				else
				{

					int		bOperations		=0;
					int		bErrors			=0;
					float	bmaxRate		=0;
					float	bmaxReadRate	=0;
					float	bmaxWriteRate	=0;
					float	bavgRate		=0; 
					float	bavgPossible	=0;
					
					//call USB2GetBenchmarkResults from USB2 DLL
					USB2GetBenchmarkResults(&bOperations, &bErrors, &bmaxRate, &bmaxReadRate, &bmaxWriteRate, &bavgRate, &bavgPossible);
					
					sprintf(tmpBuf, "%0.2f Mb/s", bmaxRate);
					SetDlgItemText(hDlg,IDC_MAX, tmpBuf);
					sprintf(tmpBuf, "%0.2f Mb/s", bavgRate);
					SetDlgItemText(hDlg,IDC_AVG, tmpBuf);

				}
			}
			else
			{
				//Loopback test has finished running
				if(Mode == LOOPBACK)
				{
					//Set the test parameters to do a benchmark test for 1 min
					//use the first connected plug (benchmark is limited to 1 test at a time)
					int firstPlug=0;
					for(; firstPlug<MAX_USB_DEV_NUMBER; firstPlug++)
					{
						if(connectedPlugs[firstPlug] == true)
						{
							break;
						}
					}

					HWND hwnd_LV_USB = GetDlgItem(g_hwndMain, IDC_PLUGLIST);
					ListView_GetItemText(hwnd_LV_USB, 0, 0, tmpBuf, 256)
					SetDlgItemText(hDlg, IDC_BENCHPLUG, tmpBuf);

					//reset use array
					memset(connectedPlugs, 0, MAX_USB_DEV_NUMBER);
					connectedPlugs[firstPlug] = true;

					//call USB2SetTestParameters from USB2 DLL
					bool ret = USB2SetTestParameters(connectedPlugs, BENCHMARK, 60, CONSTANTBYTE, 0, true);	
				
					//call USB2StartTest from USB2 DLL
					ret = USB2StartTest(); //start the benchmark test
					
					Mode = BENCHMARK;
					
					//Change status
					SetDlgItemText(hDlg, IDC_STATUS, "Started benchmark test");
					
				}
				else if(Mode == BENCHMARK)
				{
					//Bechmark test has finsihed running, kill timer
					KillTimer(hDlg, TIMER_SEC_ID);
					SetDlgItemText(hDlg, IDC_STATUS, "Finished tests");
				}
			}
			break;
		}
		
	case WM_CLOSE:
		DestroyWindow (hDlg);
		KillTimer(hDlg, TIMER_SEC_ID);
		break;
		
	case WM_COMMAND :
		{
			switch (LOWORD (wParam))
			{
			case IDEXIT:
				DestroyWindow (hDlg);
				KillTimer(hDlg, TIMER_SEC_ID);
				break;
				
			case IDABOUT:
				DialogBox (g_hInstance, "ABOUTBOX", hDlg, AboutDlgProc);
				break;
				
			} 
			break;
		}
		
	case WM_DESTROY :	
		PostQuitMessage (0) ;
		break ;

   } 
     
   return FALSE ;

 
} 

//Find the number of plugs connected and get their details
void GetPlugDetails()
{
	char	tmpBuf[256] = {""};
	char	USBSerial[50] = {"NA"};
	char	USBDesc[100] = {""};
	LVITEM	lvi;
	HWND hwnd_LV_USB = GetDlgItem(g_hwndMain, IDC_PLUGLIST);		

	//Get the number of plugs connected, call NumPlugsConnected from USB2 DLL
	int numConnectedPlugs = USB2NumPlugsConnected();

	sprintf(tmpBuf, "Detected %d USB2 Plugs", numConnectedPlugs);
	SetDlgItemText(g_hwndMain, IDC_DETECTED, tmpBuf);
	
	if(numConnectedPlugs == 0)
		return;
			
	//For all connected plugs print their serial details and description
	USB2GetConnectedPlugs(connectedPlugs);
	for(int a=0; a<MAX_USB_DEV_NUMBER; a++)
	{
		if(connectedPlugs[a] == true)
		{
			//call USB2GetDeviceInfo from USB2 DLL
			USB2GetDeviceInfo(a, USBSerial, 50, USBDesc, 100);

			//add to listview
			lvi.mask = LVIF_TEXT;
			lvi.iItem = 0;
			lvi.iSubItem =0;
			lvi.pszText = USBSerial;

			int item = ListView_InsertItem(hwnd_LV_USB, &lvi);

			ListView_SetItemText(hwnd_LV_USB, item, 1, USBDesc);
		}
	}
}


//********************************
//InitDialog
//
//Called just after the dialog is created
//********************************
void InitDialog (HWND hDlg)
{
	//Add the columns to the list view control (for detected USB2 plugs)
	LVCOLUMN	lvc;
	HWND hwnd_LV_USB = GetDlgItem(hDlg, IDC_PLUGLIST);		
	
	lvc.mask = LVCF_FMT | LVCF_WIDTH  | LVCF_TEXT | LVCF_SUBITEM ;
	lvc.fmt = LVCFMT_LEFT ;
	for (int i = 0; i < NUMUSBCOLUMNS; ++i) 
	{
		lvc.pszText = usb_columns[i];
		lvc.cchTextMax =  (int)strlen (usb_columns[i]);
		lvc.iSubItem = i;
		lvc.cx = (int) usb_columns_width[i];
		ListView_InsertColumn (hwnd_LV_USB, i, &lvc);
	}

	//Change status
	SetDlgItemText(hDlg, IDC_STATUS, "Querying USB2 devices");
	
	//Query plugs for details
	GetPlugDetails();
	
	//Set the test parameters to do a loopback test for 1 min
	//use the returned array of connected plugs (connectedPlugs) to test all of the plugs
	//call USB2SetTestParameters from USB2 DLL
	bool ret = USB2SetTestParameters(connectedPlugs, LOOPBACK, 60, CONSTANTBYTE, 0, true);

	Mode = LOOPBACK;

	//Start a loopback Test
	//call USB2StartTest from USB2 DLL
	USB2StartTest();

	//Change status
	SetDlgItemText(hDlg, IDC_STATUS, "Started loopback test");

	//Start a 1 second timer that will update the main window
	SetTimer (hDlg, TIMER_SEC_ID, 1000, NULL);

}

//
//  FUNCTION: CenterWindow(HWND, HWND)
//
//  PURPOSE:  Center one window over another.
//
//  PARAMETERS:
//    hwndChild - The handle of the window to be centered.
//    hwndParent- The handle of the window to center on.
//
//  RETURN VALUE:
//
//    TRUE  - Success
//    FALSE - Failure
//
//  COMMENTS:
//
//    Dialog boxes take on the screen position that they were designed
//    at, which is not always appropriate. Centering the dialog over a
//    particular window usually results in a better position.
//

BOOL CenterWindow(HWND hwndChild, HWND hwndParent)
{
    RECT    rcChild, rcParent;
    int     cxChild, cyChild, cxParent, cyParent;
    int     cxScreen, cyScreen, xNew, yNew;
    HDC     hdc;

    // Get the Height and Width of the child window
    GetWindowRect(hwndChild, &rcChild);
    cxChild = rcChild.right - rcChild.left;
    cyChild = rcChild.bottom - rcChild.top;

    // Get the Height and Width of the parent window
    GetWindowRect(hwndParent, &rcParent);
    cxParent = rcParent.right - rcParent.left;
    cyParent = rcParent.bottom - rcParent.top;

    // Get the display limits
    hdc = GetDC(hwndChild);
    cxScreen = GetDeviceCaps(hdc, HORZRES);
    cyScreen = GetDeviceCaps(hdc, VERTRES);
    ReleaseDC(hwndChild, hdc);

    // Calculate new X position, then adjust for screen
    xNew = rcParent.left + ((cxParent - cxChild) / 2);
    if (xNew < 0)
    {
        xNew = 0;
    }
    else if ((xNew + cxChild) > cxScreen)
    {
        xNew = cxScreen - cxChild;
    }

    // Calculate new Y position, then adjust for screen
    yNew = rcParent.top  + ((cyParent - cyChild) / 2);
    if (yNew < 0)
    {
        yNew = 0;
    }
    else if ((yNew + cyChild) > cyScreen)
    {
        yNew = cyScreen - cyChild;
    }

    // Set it, and return
    return SetWindowPos(hwndChild,
                        NULL,
                        xNew, yNew,
                        0, 0,
                        SWP_NOSIZE | SWP_NOZORDER);
}



INT_PTR CALLBACK AboutDlgProc (HWND hDlg, UINT iMsg, WPARAM wParam, LPARAM lParam)
{
	
	char				tstr[MAXSTRLEN];
	
	switch (iMsg)
	{
	case WM_INITDIALOG :
		
		// Center the dialog over the application window
		CenterWindow(hDlg, g_hwndMain);
		
		sprintf (tstr, "%s Build: %s", PROGRAM_VERSION, PROGRAM_BUILD);
		SetDlgItemText (hDlg, IDC_VERSIONNUM, tstr);
		
		return TRUE ;
		
	case WM_COMMAND :
		switch (LOWORD (wParam))
		{
		case IDOK :
		case IDCANCEL :
			EndDialog (hDlg, 0) ;
			return TRUE ;
		}
		break ;
	}
	return FALSE ;
}



